@@ -1,4 +1,4 @@
-/*  $Id: inews.c,v 1.54.2.2 2004/12/19 22:15:41 rra Exp $
+/*  $Id: inews.c 7769 2008-04-13 08:11:41Z iulius $
 **
 **  Send an article (prepared by someone on the local site) to the
 **  master news server.
@@ -989,8 +989,12 @@ main(int ac, char *av[])
 	if ((j = atoi(buff)) != NNTP_BAD_COMMAND_VAL)
 	    i = j;
 
-	if (i != NNTP_POSTOK_VAL)
-            die("you do not have permission to post");
+        if (i != NNTP_POSTOK_VAL) {
+            /* We try to authenticate in case it is all the same possible
+             * to post. */
+            if (NNTPsendpassword((char *)NULL, FromServer, ToServer) < 0)
+                die("you do not have permission to post");
+        }
 	deadfile = NULL;
     }
 
