@@ -1,15 +1,14 @@
-/*  $Id: ng.c,v 1.21 1999/11/19 08:59:20 rra Exp $
+/*  $Id: ng.c 6494 2003-10-20 01:12:50Z rra $
 **
 **  Routine for the in-core data structures for the active and newsfeeds
 **  files.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include <sys/mman.h>
-#include <dirent.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
+#include <dirent.h>
+
+#include "inn/innconf.h"
 #include "innd.h"
 #include "ov.h"
 
@@ -40,11 +39,10 @@ typedef struct _NGHASH {
 } NGHASH;
 
 
-STATIC BUFFER	NGdirs;
-STATIC BUFFER	NGnames;
-STATIC NGHASH	NGHtable[NGH_SIZE];
-STATIC int	NGHbuckets;
-STATIC int	NGHcount;
+static struct buffer	NGnames;
+static NGHASH		NGHtable[NGH_SIZE];
+static int		NGHbuckets;
+static int		NGHcount;
 
 
 
@@ -53,30 +51,11 @@ STATIC int	NGHcount;
 **  rough order of their activity.  Will be better if we write a "counts"
 **  file sometime.
 */
-STATIC int
-NGcompare(p1, p2)
-    CPOINTER p1;
-    CPOINTER p2;
+static int
+NGcompare(const void *p1, const void *p2)
 {
-    NEWSGROUP	**ng1;
-    NEWSGROUP	**ng2;
-
-    ng1 = (NEWSGROUP **) p1;
-    ng2 = (NEWSGROUP **) p2;
-    return ng1[0]->Last - ng2[0]->Last;
-}
-
-
-/*
-**  Convert a newsgroup name into a directory name.
-*/
-STATIC void
-NGdirname(p)
-    register char	*p;
-{
-    for ( ; *p; p++)
-	if (*p == '.')
-	    *p = '/';
+    return ((const NEWSGROUP **)p1)[0]->Last -	
+	   ((const NEWSGROUP **)p2)[0]->Last;
 }
 
 
@@ -85,68 +64,62 @@ NGdirname(p)
 **  not to write NUL's into the in-core copy, since we're either mmap(2)'d,
 **  or we want to just blat it out to disk later.
 */
-STATIC BOOL NGparseentry(NEWSGROUP *ngp, char *p, char *end)
+static bool
+NGparseentry(NEWSGROUP *ngp, const char *p, char *end)
 {
-    register char		*q;
-    register unsigned int	j;
-    register NGHASH		*htp;
-    register NEWSGROUP		**ngpp;
-    register int		i;
-    ARTNUM			lo;
+    char		*q;
+    unsigned int	j;
+    NGHASH		*htp;
+    NEWSGROUP		**ngpp;
+    int			i;
+    ARTNUM		lo;
 
     if ((q = strchr(p, ' ')) == NULL)
-	return FALSE;
+	return false;
     i = q - p;
 
     ngp->NameLength = i;
-    ngp->Name = &NGnames.Data[NGnames.Used];
-    (void)strncpy(ngp->Name, p, (SIZE_T)i);
+    ngp->Name = &NGnames.data[NGnames.used];
+    strncpy(ngp->Name, p, i);
     ngp->Name[i] = '\0';
-    NGnames.Used += i + 1;
-
-    ngp->Dir = &NGdirs.Data[NGdirs.Used];
-    (void)strncpy(ngp->Dir, p, (SIZE_T)i);
-    ngp->Dir[i] = '\0';
-    NGdirs.Used += i + 1;
-    NGdirname(ngp->Dir);
+    NGnames.used += i + 1;
 
     ngp->LastString = ++q;
     if ((q = strchr(q, ' ')) == NULL || q > end)
-	return FALSE;
+	return false;
     ngp->Lastwidth = q - ngp->LastString;
     if ((q = strchr(q, ' ')) == NULL || q > end)
-	return FALSE;
+	return false;
     lo = (ARTNUM)atol(q + 1);
     if ((q = strchr(q + 1, ' ')) == NULL || q > end)
-	return FALSE;
+	return false;
     ngp->Rest = ++q;
     /* We count on atoi() to stop at the space after the digits! */
     ngp->Last = atol(ngp->LastString);
     ngp->nSites = 0;
-    ngp->Sites = NEW(int, NGHcount);
+    ngp->Sites = xmalloc(NGHcount * sizeof(int));
     ngp->nPoison = 0;
-    ngp->Poison = NEW(int, NGHcount);
+    ngp->Poison = xmalloc(NGHcount * sizeof(int));
     ngp->Alias = NULL;
 
     /* Find the right bucket for the group, make sure there is room. */
-    /* SUPPRESS 6 *//* Over/underflow from plus expression */
     NGH_HASH(ngp->Name, p, j);
     htp = NGH_BUCKET(j);
     for (p = ngp->Name, ngpp = htp->Groups, i = htp->Used; --i >= 0; ngpp++)
-	if (*p == ngpp[0]->Name[0] && EQ(p, ngpp[0]->Name)) {
+	if (*p == ngpp[0]->Name[0] && strcmp(p, ngpp[0]->Name) == 0) {
 	    syslog(L_ERROR, "%s duplicate_group %s", LogName, p);
-	    return FALSE;
+	    return false;
 	}
     if (htp->Used >= htp->Size) {
 	htp->Size += NGHbuckets;
-	RENEW(htp->Groups, NEWSGROUP*, htp->Size);
+        htp->Groups = xrealloc(htp->Groups, htp->Size * sizeof(NEWSGROUP *));
     }
     htp->Groups[htp->Used++] = ngp;
 
     if (innconf->enableoverview && !OVgroupadd(ngp->Name, lo, ngp->Last, ngp->Rest))
-	return FALSE;
+	return false;
 
-    return TRUE;
+    return true;
 }
 
 
@@ -154,17 +127,17 @@ STATIC BOOL NGparseentry(NEWSGROUP *ngp, char *p, char *end)
 **  Parse the active file, building the initial Groups global.
 */
 void
-NGparsefile()
+NGparsefile(void)
 {
-    register char	*p;
-    register char	*q;
-    register int	i;
-    register BOOL	SawMe;
-    register NEWSGROUP	*ngp;
-    register NGHASH	*htp;
-    char		**strings;
-    char		*active;
-    char		*end;
+    char	*p;
+    char	*q;
+    int		i;
+    bool	SawMe;
+    NEWSGROUP	*ngp;
+    NGHASH	*htp;
+    char	**strings;
+    char	*active;
+    char	*end;
 
     /* If re-reading, remove anything we might have had. */
     NGclose();
@@ -177,19 +150,16 @@ NGparsefile()
 	syslog(L_FATAL, "%s empty active file", LogName);
 	exit(1);
     }
-    Groups = NEW(NEWSGROUP, nGroups);
-    GroupPointers = NEW(NEWSGROUP*, nGroups);
+    Groups = xmalloc(nGroups * sizeof(NEWSGROUP));
+    GroupPointers = xmalloc(nGroups * sizeof(NEWSGROUP *));
 
-    /* Get space to hold copies of the names and the directory names.
-     * This might take more space than individually allocating each
-     * element, but it is definitely easier on the system. */
+    /* Get space to hold copies of the names.  This might take more space
+     * than individually allocating each element, but it is definitely easier
+     * on the system. */
     i = end - active;
-    NGdirs.Size = i;
-    NGdirs.Data = NEW(char, NGdirs.Size + 1);
-    NGdirs.Used = 0;
-    NGnames.Size = i;
-    NGnames.Data = NEW(char, NGnames.Size + 1);
-    NGnames.Used = 0;
+    NGnames.size = i;
+    NGnames.data = xmalloc(NGnames.size + 1);
+    NGnames.used = 0;
 
     /* Set up the default hash buckets. */
     NGHbuckets = nGroups / NGH_SIZE;
@@ -201,15 +171,15 @@ NGparsefile()
     else
 	for (i = NGH_SIZE, htp = NGHtable; --i >= 0; htp++) {
 	    htp->Size = NGHbuckets;
-	    htp->Groups = NEW(NEWSGROUP*, htp->Size);
+	    htp->Groups = xmalloc(htp->Size * sizeof(NEWSGROUP *));
 	    htp->Used = 0;
 	}
 
     /* Count the number of sites. */
-    SawMe = FALSE;
-    for (strings = SITEreadfile(TRUE), i = 0; (p = strings[i]) != NULL; i++)
+    SawMe = false;
+    for (strings = SITEreadfile(true), i = 0; (p = strings[i]) != NULL; i++)
 	if (*p == 'M' && *++p == 'E' && *++p == ':')
-	    SawMe = TRUE;
+	    SawMe = true;
     if (i == 0 || (i == 1 && SawMe)) {
 	syslog(L_ERROR, "%s bad_newsfeeds no feeding sites", LogName);
 	NGHcount = 1;
@@ -230,8 +200,7 @@ NGparsefile()
     /* Sort each bucket. */
     for (i = NGH_SIZE, htp = NGHtable; --i >= 0; htp++)
 	if (htp->Used > 1)
-	    qsort((POINTER)htp->Groups, (SIZE_T)htp->Used,
-		sizeof htp->Groups[0], NGcompare);
+	    qsort(htp->Groups, htp->Used, sizeof htp->Groups[0], NGcompare);
 
     /* Chase down any alias flags. */
     for (ngp = Groups, i = nGroups; --i >= 0; ngp++)
@@ -252,28 +221,27 @@ NGparsefile()
 ** Free allocated memory
 */
 void
-NGclose()
+NGclose(void)
 {
-    register int	i;
-    register NEWSGROUP	*ngp;
-    register NGHASH	*htp;
+    int		i;
+    NEWSGROUP	*ngp;
+    NGHASH	*htp;
 
     if (Groups) {
 	for (i = nGroups, ngp = Groups; --i >= 0; ngp++) {
-	    DISPOSE(ngp->Sites);
-	    DISPOSE(ngp->Poison);
+	    free(ngp->Sites);
+	    free(ngp->Poison);
 	}
-	DISPOSE(Groups);
+	free(Groups);
 	Groups = NULL;
-	DISPOSE(GroupPointers);
-	DISPOSE(NGdirs.Data);
-	DISPOSE(NGnames.Data);
+	free(GroupPointers);
+	free(NGnames.data);
     }
 
     for (i = NGH_SIZE, htp = NGHtable; --i >= 0; htp++) {
       htp->Size = NGHbuckets;
       if (htp->Groups) {
-	DISPOSE(htp->Groups);
+	free(htp->Groups);
 	htp->Used = 0;
 	htp->Groups = NULL;
       }
@@ -284,66 +252,69 @@ NGclose()
 **  Hash a newsgroup and see if we get it.
 */
 NEWSGROUP *
-NGfind(Name)
-    char			*Name;
+NGfind(const char *Name)
 {
-    register char		*p;
-    register int		i;
-    register unsigned int	j;
-    register NEWSGROUP		**ngp;
-    char			c;
-    NGHASH			*htp;
+    const char		*p;
+    int			i;
+    unsigned int	j;
+    NEWSGROUP		**ngp;
+    char		c;
+    NGHASH		*htp;
 
-    /* SUPPRESS 6 *//* Over/underflow from plus expression */
     NGH_HASH(Name, p, j);
     htp = NGH_BUCKET(j);
     for (c = *Name, ngp = htp->Groups, i = htp->Used; --i >= 0; ngp++)
-	if (c == ngp[0]->Name[0] && EQ(Name, ngp[0]->Name))
+	if (c == ngp[0]->Name[0] && strcmp(Name, ngp[0]->Name) == 0)
 	    return ngp[0];
     return NULL;
 }
 
 
 /*
-**  Split a newsgroups header line into the groups we get.  Return a
-**  point to static memory and clobber the argument along the way.
+**  Split a newsgroups header line into the groups we get.  Return the
+**  number of newsgroups.  ' ' and '\t' are dropped when copying.
 */
-char **
-NGsplit(p)
-    register char	*p;
+int
+NGsplit(char *p, int size, LISTBUFFER *list)
 {
-    static char		**groups;
-    static int		oldlength;
-    register char	**gp;
-    register int	i;
+  char		**gp, *q;
+  int		i;
 
-    /* Get an array of character pointers. */
-    i = strlen(p);
-    if (groups == NULL) {
-	groups = NEW(char*, i + 1);
-	oldlength = i;
-    }
-    else if (oldlength < i) {
-	RENEW(groups, char*, i + 1);
-	oldlength = i;
-    }
+  /* setup buffer */
+  SetupListBuffer(size, list);
 
-    /* Loop over text. */
-    for (gp = groups; *p; *p++ = '\0') {
-	/* Skip leading separators. */
-	for (; NG_ISSEP(*p); p++)
-	    continue;
-	if (*p == '\0')
-	    break;
+  /* loop over and copy */
+  for (i = 0, q = list->Data, gp = list->List ; *p ; p++, *q++ = '\0') {
+    /* skip leading separators and white spaces. */
+    for (; *p && (NG_ISSEP(*p) || ISWHITE(*p)) ; p++)
+      continue;
+    if (*p == '\0')
+      break;
 
-	/* Mark the start of the newsgroup, move to the end of it. */
-	for (*gp++ = p; *p && !NG_ISSEP(*p); p++)
-	    continue;
-	if (*p == '\0')
-	    break;
+    if (i == list->ListLength) {
+      list->ListLength += DEFAULTNGBOXSIZE;
+      list->List = xrealloc(list->List, list->ListLength * sizeof(char *));
+      gp = &list->List[i];
     }
-    *gp = NULL;
-    return groups;
+    /* mark the start of the newsgroup, move to the end of it while copying */
+    for (*gp++ = q, i++ ; *p && !NG_ISSEP(*p) && !ISWHITE(*p) ;) {
+      if (*p == ':')
+	/* reject if ':' is included */
+	return 0;
+      *q++ = *p++;
+      continue;
+    }
+    if (*p == '\0')
+      break;
+  }
+  *q = '\0';
+  if (i == list->ListLength) {
+    list->ListLength += DEFAULTNGBOXSIZE;
+    list->List = xrealloc(list->List, list->ListLength * sizeof(char *));
+    gp = &list->List[i];
+  }
+  *gp = NULL;
+  return i;
 }
 
 
@@ -353,7 +324,8 @@ NGsplit(p)
 static char		NORENUMBER[] = "%s cant renumber %s %s too wide";
 static char		RENUMBER[] = "%s renumber %s %s from %ld to %ld";
 
-BOOL NGrenumber(NEWSGROUP *ngp)
+bool
+NGrenumber(NEWSGROUP *ngp)
 {
     int			low, high, count,flag;
     char	        *f2;
@@ -364,12 +336,12 @@ BOOL NGrenumber(NEWSGROUP *ngp)
     long		l;
     char		*dummy;
 
-    if (!innconf->enableoverview) return TRUE; /* can't do anything w/o overview */
+    if (!innconf->enableoverview) return true; /* can't do anything w/o overview */
 
     /* Get a valid offset into the active file. */
     if (ICDneedsetup) {
 	syslog(L_ERROR, "%s unsynched must reload before renumber", LogName);
-	return FALSE;
+	return false;
     }
     start = ICDreadactive(&dummy) + ngp->Start;
 
@@ -379,28 +351,31 @@ BOOL NGrenumber(NEWSGROUP *ngp)
      || (f4 = strchr(++f3, ' ')) == NULL) {
 	syslog(L_ERROR, "%s bad_format active %s",
 	    LogName, MaxLength(start, start));
-	return FALSE;
+	return false;
     }
     himark = atol(f2);
     lomark = himark + 1;
     /* note these will be the low and himarks if the group turns out to be empty. */
 
     /* Check overview data for the group. */
-    if (!OVgroupstats(ngp->Name, &low, &high, &count, &flag)) return FALSE;
+    if (!OVgroupstats(ngp->Name, &low, &high, &count, &flag)) return false;
     if (count != 0) {
 	/* non-empty group, so set low/himarks from overview. */
 	lomark = low;
 	himark = high;
     }
     l = atol(f2);
-    if (himark != l) {
+    if (himark > l) {
 	syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "hi", l, himark);
 	if (!FormatLong(f2, himark, f3 - f2 - 1)) {
 	    syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "hi");
-	    return FALSE;
+	    return false;
 	}
 	ngp->Last = himark;
 	ICDactivedirty++;
+    } else if (himark < l) {
+        syslog(L_NOTICE, "%s renumber %s hi not decreasing %ld to %ld",
+               LogName, ngp->Name, l, himark);
     }
     l = atol(f3);
     if (lomark != l) {
@@ -408,11 +383,11 @@ BOOL NGrenumber(NEWSGROUP *ngp)
 	    syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "lo", l, lomark);
 	if (!FormatLong(f3, lomark, f4 - f3)) {
 	    syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "lo");
-	    return FALSE;
+	    return false;
 	}
 	ICDactivedirty++;
     }
-    return TRUE;
+    return true;
 }
 
 /*
@@ -420,7 +395,8 @@ BOOL NGrenumber(NEWSGROUP *ngp)
  * Like NGrenumber(), but we don't scan the spool,
  * and the himark is ignored.
  */
-BOOL NGlowmark(NEWSGROUP *ngp, long lomark)
+bool
+NGlowmark(NEWSGROUP *ngp, long lomark)
 {
     long l;
     char *f2, *f3, *f4;
@@ -433,7 +409,7 @@ BOOL NGlowmark(NEWSGROUP *ngp, long lomark)
      || (f4 = strchr(++f3, ' ')) == NULL) {
         syslog(L_ERROR, "%s bad_format active %s",
             LogName, MaxLength(start, start));
-        return FALSE;
+        return false;
     }
     l = atol(f3);
     if (lomark != l) {
@@ -441,9 +417,9 @@ BOOL NGlowmark(NEWSGROUP *ngp, long lomark)
             syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "lo", l, lomark);
         if (!FormatLong(f3, lomark, f4 - f3)) {
             syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "lo");
-            return FALSE;
+            return false;
         }
         ICDactivedirty++;
     }
-    return TRUE;
+    return true;
 }
