@@ -1,4 +1,4 @@
-/*  $Id: python.c,v 1.23 2003/02/11 06:48:55 alexk Exp $
+/*  $Id: python.c 6650 2004-01-25 22:22:14Z rra $
 **
 **  python.c: Embed Python in the style of innd's TCL and Perl stuff.
 ** 
@@ -645,6 +645,7 @@ PYsetup(void)
 {
     const ARTHEADER *hp;
     int		hdrindex;
+    size_t hdrcount;
 
     setenv("PYTHONPATH", innconf->pathfilter, 1);
     Py_Initialize();
@@ -675,10 +676,13 @@ PYsetup(void)
 	syslog(L_NOTICE, "defined python methods");
     }
 
-    /* Grab space for these so we aren't forever recreating them. */
+    /* Grab space for these so we aren't forever recreating them.  We also
+       put the body and the line count into PYheaditem, so it needs to be
+       two elements longer than the total number of headers. */
     PYheaders = PyDict_New();
-    PYheaditem = xmalloc((ARRAY_END(ARTheaders) - ARTheaders) * sizeof(PyObject *));
-    PYheadkey = xmalloc((ARRAY_END(ARTheaders) - ARTheaders) * sizeof(PyObject *));
+    hdrcount = ARRAY_END(ARTheaders) - ARTheaders;
+    PYheaditem = xmalloc((hdrcount + 2) * sizeof(PyObject *));
+    PYheadkey = xmalloc(hdrcount * sizeof(PyObject *));
 
     /* Preallocate keys for the article dictionary */
     for (hp = ARTheaders; hp < ARRAY_END(ARTheaders); hp++)
