@@ -1,4 +1,4 @@
-/*  $Id: perl.c 6135 2003-01-19 01:15:40Z rra $
+/*  $Id: perl.c 7815 2008-05-05 08:43:58Z iulius $
 **
 **  Embedded Perl support for INN.
 **
@@ -62,6 +62,7 @@ bool
 PerlFilter(bool value)
 {
     dSP;
+    char *argv[] = { NULL };
 
     if (value == PerlFilterActive)
         return true;
@@ -71,7 +72,7 @@ PerlFilter(bool value)
         if (perl_get_cv("filter_end", false) != NULL) {
             ENTER;
             SAVETMPS;
-            perl_call_argv("filter_end", G_EVAL | G_DISCARD | G_NOARGS, NULL);
+            perl_call_argv("filter_end", G_EVAL | G_DISCARD | G_NOARGS, argv);
             if (SvTRUE(ERRSV)) {
                 syslog (L_ERROR, "SERVER perl function filter_end died: %s",
                         SvPV(ERRSV, PL_na));
@@ -172,11 +173,10 @@ int PERLreadfilter(char *filterfile, const char *function)
     ENTER ;
     SAVETMPS ;
     
-    argv[0] = filterfile ;
-    argv[1] = NULL ;
-    
+    argv[0] = NULL;
+
     if (perl_get_cv("filter_before_reload", false) != NULL)    {
-        perl_call_argv("filter_before_reload",G_EVAL|G_DISCARD|G_NOARGS,NULL);
+        perl_call_argv("filter_before_reload", G_EVAL|G_DISCARD|G_NOARGS, argv);
         if (SvTRUE(ERRSV))     /* check $@ */ {
             syslog (L_ERROR,"SERVER perl function filter_before_reload died: %s",
                     SvPV(ERRSV, PL_na)) ;
@@ -185,6 +185,9 @@ int PERLreadfilter(char *filterfile, const char *function)
         }
     }
 
+    argv[0] = filterfile ;
+    argv[1] = NULL ;
+    
     PerlSilence();
     perl_call_argv ("_load_", 0, argv) ;
     PerlUnSilence();
@@ -209,8 +212,9 @@ int PERLreadfilter(char *filterfile, const char *function)
         PerlFilter (false) ;
     }
     
+    argv[0] = NULL;
     if (perl_get_cv("filter_after_reload", false) != NULL) {
-        perl_call_argv("filter_after_reload", G_EVAL|G_DISCARD|G_NOARGS, NULL);
+        perl_call_argv("filter_after_reload", G_EVAL|G_DISCARD|G_NOARGS, argv);
         if (SvTRUE(ERRSV))     /* check $@ */ {
             syslog (L_ERROR,"SERVER perl function filter_after_reload died: %s",
                     SvPV(ERRSV, PL_na)) ;
