@@ -241,16 +241,39 @@ sub collect {
       return 1 if $left =~ /^SERVER perl filtering enabled$/o;
       # SERVER perl filtering disabled
       return 1 if $left =~ /^SERVER perl filtering disabled$/o;
+      # SERVER Python filtering enabled
+      return 1 if $left =~ /^SERVER Python filtering enabled$/o;
+      # SERVER Python filtering disabled
+      return 1 if $left =~ /^SERVER Python filtering disabled$/o;
       # SERVER cancelled +id
       return 1 if $left =~ /^SERVER cancelled /o;
     }
+    # Python filter
+    return 1 if $left =~ /^defined python methods$/o;
+    return 1 if $left =~ /^python interpreter initialized OK$/o;
+    return 1 if $left =~ /^python: First load, so I can do initialization stuff.$/o;
+    return 1 if $left =~ /^python: spamfilter successfully hooked into INN$/o;
+    return 1 if $left =~ /^python: filter_close running, bye!$/o;
     # rejecting[perl]
     if ($left =~ /^rejecting\[perl\] <[^>]+> \d+ (.*)/o) {
       $innd_filter_perl{$1}++;
       return 1;
     }
+    # rejecting[python]
+    if ($left =~ /^rejecting\[python\] <[^>]+> \d+ (.*)/o) {
+      $innd_filter_python{$1}++;
+      return 1;
+    }
     # closed lost
     return 1 if $left =~ /^\S+ closed lost \d+/o;
+    # new control command
+    if ($left =~ /^ctlinnd command (\w)(:.*)?/o) {
+      my $command = $1;
+      my $cmd = $ctlinnd{$command};
+      $cmd = $command unless $cmd;
+      return 1 if $cmd eq 'flush'; # to avoid a double count
+      $innd_control{"$cmd"}++;
+    }
     # control command (by letter)
     if ($left =~ /^(\w)$/o) {
       my $command = $1;
@@ -326,7 +349,7 @@ sub collect {
       return 1;
     }
     # closed (with times)
-    if ($left =~ /(\S+):\d+ closed seconds (\d+) accepted (\d+) refused (\d+) rejected (\d+) duplicate (\d+) accepted size (\d+) duplicate size (\d+)$/o) {
+    if ($left =~ /(\S+):\d+ closed seconds (\d+) accepted (\d+) refused (\d+) rejected (\d+) duplicate (\d+) accepted size (\d+) duplicate size (\d+)(?: rejected size (\d+))?$/o) {
       my ($server, $seconds, $accepted, $refused, $rejected, $duplicate, $accptsize, $dupsize) =
 	($1, $2, $3, $4, $5, $6, $7, $8);
       $server = lc $server unless $CASE_SENSITIVE;
@@ -350,6 +373,8 @@ sub collect {
     }
     # closed (without times (?))
     return 1 if $left =~ m/\S+ closed$/o;
+    # closed (for a cancel feed - MODE CANCEL)
+    return 1 if $left =~ m/localhost:\d+ closed seconds \d+ cancels \d+$/o;
     # checkpoint
     return 1 if $left =~ m/^\S+:\d+ checkpoint /o;
     # if ($left =~ /(\S+):\d+ checkpoint seconds (\d+) accepted (\d+)
@@ -1212,6 +1237,8 @@ sub collect {
     return 1 if $left =~ /\S+ cant opendir \S+ I\/O error$/o;
     # perl filtering enabled
     return 1 if $left =~ /perl filtering enabled$/o;
+    # Python filtering enabled
+    return 1 if $left =~ /Python filtering enabled$/o;
     # connect
     if ($left =~ /(\S+) (\([0-9a-fA-F:.]*\) )?connect$/o) {
       my $cust = $1;
@@ -1553,6 +1580,16 @@ sub collect {
     return 1 if $left =~ m/rejected 437 ECP rejected/o;
     # rejected 437 "Subject" header too long
     return 1 if $left =~ m/header too long/o;
+    # rejected 437 Too long line in header 1163 bytes
+    return 1 if $left =~ m/rejected 437 Too long line in header/o;
+    # rejected 437 Too many newsgroups (meow)
+    return 1 if $left =~ m/rejected 437 Too many newsgroups/o;
+    # rejected 437 Space before colon in "<a" header
+    return 1 if $left =~ m/rejected 437 Space before colon in/o;
+    # rejected 437 EMP (phl)
+    return 1 if $left =~ m/rejected 437 EMP/o;
+    # rejected 437 Scoring filter (8)
+    return 1 if $left =~ m/rejected 437 Scoring filter/o;
     # bad_article missing Message-ID
     return 1 if $left =~ m/bad_article missing Message-ID/o;
     # cant unspool saving to xxx
@@ -1588,6 +1625,29 @@ sub collect {
     return 1;
   }
 
+  ########
+  ## nocem
+  if ($prog eq "nocem") {
+    if ($left =~ /processed notice .* by (.*) \((\d+) ids,/o) {
+      $nocem_goodsigs{$1}++;
+      $nocem_totalgood++;
+      $nocem_lastid = $1;
+      $nocem_newids += $2;
+      $nocem_newids{$nocem_lastid} += $2;
+      $nocem_totalids += $2;
+      $nocem_totalids{$nocem_lastid} += $2;
+      return 1;
+    }
+    if ($left =~ /bad signature from (.*)/o) {
+      $nocem_badsigs{$1}++;
+      $nocem_goodsigs{$1} = 0 unless ($nocem_goodsigs{$1});
+      $nocem_totalbad++;
+      $nocem_lastid = $1;
+      return 1;
+    }
+    return 1;
+  }
+
   ###########
   ## controlchan
   if ($prog eq "controlchan") {
