@@ -1,4 +1,4 @@
-/* $Id: hstrerror-t.c 7619 2007-08-01 20:23:12Z iulius $ */
+/* $Id: hstrerror-t.c 5060 2001-12-12 09:20:10Z rra $ */
 /* hstrerror test suite. */
 
 #include "config.h"
@@ -18,13 +18,15 @@ test_error(int n, const char *expected, int error)
 int
 main(void)
 {
-    test_init(5);
+    puts("7");
 
     test_error(1, "Internal resolver error", -1);
     test_error(2, "No resolver error", 0);
     test_error(3, "No address associated with name", NO_ADDRESS);
     test_error(4, "Resolver error 777777", 777777);
     test_error(5, "Resolver error -99999", -99999);
+    test_error(6, "", 1000000);
+    test_error(7, "", -100000);
 
     return 0;
 }
